SystemOrganization addCategory: #'RPackage-Core'!

TestCase subclass: #RPackage2IncrementalTest
	instanceVariableNames: 'createdClasses createdPackages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackage2IncrementalTest commentStamp: 'StephaneDucasse 5/15/2010 20:21' prior: 0!
The tests takes as fixture the following situation (which is constructed partially)
For tests that do not need to check the incremental construction.


P1 
	A1DefinedInP1>>methodDefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
P2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (was *P2*)!

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 2/21/2010 17:29'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackage2Test'.
	createdClasses add: cls.
	^ cls! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 2/21/2010 17:29'!
createNewClassNamed: aName inCategory: cat
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: cat.
	createdClasses add: cls.
	^ cls! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 3/11/2010 13:28'!
createNewClassNamed: aName inPackage: p
	
	| cls |
	cls := self createNewClassNamed: aName.
	p addClassDefinition: cls.
	^ cls! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 5/15/2010 20:28'!
createNewPackageNamed: aName

	| pack |
	self removePackage: aName.
	pack := self packageClass named: aName.
	createdPackages add: pack.
	^ pack! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 2/21/2010 17:32'!
packageClass

	^ RPackage2! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'StephaneDucasse 2/21/2010 17:34'!
removeClassNamedIfExists: aClassNameSymbol

	Smalltalk at: aClassNameSymbol asSymbol ifPresent: [:c| c removeFromSystem]! !

!RPackage2IncrementalTest methodsFor: 'utility' stamp: 'sd 5/19/2010 16:42'!
removePackage: aName

	self packageClass packageOrganizerClass default unregisterPackageNamed: aName.
	"SystemOrganization removeCategory: aName."! !

!RPackage2IncrementalTest methodsFor: 'setup' stamp: 'sd 5/19/2010 16:17'!
setUp

	self packageClass packageOrganizerClass: RMockEmptyPackageOrganizer.
	createdClasses := OrderedCollection new.
	createdPackages := OrderedCollection new.! !

!RPackage2IncrementalTest methodsFor: 'setup' stamp: 'sd 5/19/2010 16:13'!
tearDown

	[
	createdPackages  do: [:each | self removePackage: each name].
	createdPackages  do: [:each | 
					each extendedClasses do: [:extendedClass|
							self packageClass packageOrganizerClass default
						 		unregisterExtendingPackage: each forClass: extendedClass]].
						
	(createdClasses reject: [:c| c isObsolete])
		do: [:cls| 
				(self packageClass packageOrganizerClass default includesClass: cls)
					ifTrue: [self packageClass packageOrganizerClass default unregisterClass: cls.].
				cls removeFromSystem ].					
						
			]
			ensure: [ self packageClass packageOrganizerClass: RPackageOrganizer].
			
	SystemOrganization removeCategory: 'RPackage2Test'! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'sd 4/24/2010 22:39'!
testAddClassAfterMethods
	"self debug: #testAddClassAfterMethods"

	| p1 a1 b1 |
	p1 := self createNewPackageNamed: 'P1'.
	"the class is created but not added to the package for now"
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.
	self assert: p1 definedClasses size = 0.
	
	"now we add a new method to the class and the to the package: it is considered
	as class extension"
	
	a1 compileSilently: 'newlyAddedToA1 ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#newlyAddedToA1).
	self assert: (p1 includesSelector: #newlyAddedToA1 ofClass: a1).
	self assert: (p1 includesExtensionSelector: #newlyAddedToA1 ofClass: a1).
	self deny: (p1 includesDefinedSelector: #newlyAddedToA1 ofClass: a1).
	
	"Now we add the class and this means that the method is not an extension anymore but 
	a defined method"
	
	p1 addClassDefinition: a1.
	self assert: p1 definedClasses size = 1.
	
	self deny: (p1 includesExtensionSelector: #newlyAddedToA1 ofClass: a1).
	self assert: (p1 includesDefinedSelector: #newlyAddedToA1 ofClass: a1).
	
	
"
	self assert: (p1 includesSelector: #methodDefinedInP1 ofClass: a1).
	self assert: (p3 includesSelector: #classSideMethodDefinedInP3 ofClass: a2 class)
	"! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'sd 4/25/2010 14:14'!
testAddClassAfterMethodsAtMetaclassToo
	"self debug: #testAddClassAfterMethodsAtMetaclassToo"

	| p1 a1 b1 |
	p1 := self createNewPackageNamed: 'P1'.
	"the class is created but not added to the package for now"
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.
	self assert: p1 definedClasses size = 0.
	
	"now we add a new method to the class and the to the package: it is considered
	as class extension"
	
	a1 class compileSilently: 'newlyAddedToA1Class ^ #methodDefinedInP1'.
	p1 addMethod: (a1 class>>#newlyAddedToA1Class).
	self assert: (p1 includesSelector: #newlyAddedToA1Class ofClass: a1 class).
	self assert: (p1 includesExtensionSelector: #newlyAddedToA1Class ofClass: a1 class).
	self deny: (p1 includesDefinedSelector: #newlyAddedToA1Class ofClass: a1 class).
	
	"Now we add the class and this means that the method is not an extension anymore but 
	a defined method"
	
	p1 addClassDefinition: a1.
	self assert: p1 definedClasses size = 1.
	
	self deny: (p1 includesExtensionSelector: #newlyAddedToA1Class ofClass: a1 class).
	self assert: (p1 includesDefinedSelector: #newlyAddedToA1Class ofClass: a1 class).
	
	
"
	self assert: (p1 includesSelector: #methodDefinedInP1 ofClass: a1).
	self assert: (p3 includesSelector: #classSideMethodDefinedInP3 ofClass: a2 class)
	"! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 15:10'!
testAddClassDefinitionNoDuplicate
	"self debug: #testAddClassDefinitionNoDuplicate"

	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	self assert: p definedClasses size = 1.
	b1 := self createNewClassNamed: #B1InPackageP1 inCategory: 'P1'.
	p addClassDefinition: a1.
	"adding the same class does not do anything - luckily"
	self assert: p definedClasses size = 1.
	p addClassDefinition: b1.
	self assert: p definedClasses size = 2.
! !

!RPackage2IncrementalTest methodsFor: 'test method addition removal' stamp: 'StephaneDucasse 4/21/2010 08:52'!
testAddRemoveMethod
	"self debug: #testAddRemoveMethod"
	
	| p1 p2 p3 a2   a2Name |
	a2Name := #A2InPackageP2.
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: a2Name inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	
	p2 addMethod: (a2>>#methodDefinedInP2).
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
	
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2Name).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2Name).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofClassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofClassName: a2Name).
	
	"removeMethod by default removes from defined methods and not extension"
	p2 removeMethod: (a2>>#methodDefinedInP2).
	self deny: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofClassName: a2Name).
	
	p1 removeMethod: (a2>>#methodDefinedInP1).
	self deny: (p1 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2Name).
	self deny: (p1 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2Name).
	
	
	


! !

!RPackage2IncrementalTest methodsFor: 'test method addition removal' stamp: 'StephaneDucasse 4/20/2010 22:56'!
testAddRemoveSelector
	"self debug: #testAddRemoveSelector"
	
	| p1 p2 p3 a2   a2Name |
	a2Name := #A2InPackageP2.
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: a2Name inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addSelector: #methodDefinedInP2 ofClassName: a2Name.
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addSelector: #methodDefinedInP1 ofClassName: a2Name.
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addSelector: #methodDefinedInP3 ofClassName: a2Name.
	
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2Name).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2Name).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofClassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofClassName: a2Name).
	
	p2 removeSelector: #methodDefinedInP2 ofClassName:  a2Name.
	self deny: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofClassName: a2Name).
	
	p1 removeSelector: #methodDefinedInP1 ofClassName:  a2Name.
	self deny: (p1 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2Name).
	self deny: (p1 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2Name).
	
	
	


! !

!RPackage2IncrementalTest methodsFor: 'test method addition removal' stamp: 'StephaneDucasse 4/20/2010 22:59'!
testAddRemoveSelectorOfMetaclass
	"self debug: #testAddRemoveSelectorOfMetaclass"
	
	| p1 p2 p3 a2   a2Name a2class |
	a2Name := #A2InPackageP2.
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: a2Name inPackage: p2.
	a2class := a2 class.
	a2class compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addSelector: #methodDefinedInP2 ofMetaclassName: a2Name.
	a2class compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addSelector: #methodDefinedInP1 ofMetaclassName: a2Name.
	a2class compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addSelector: #methodDefinedInP3 ofMetaclassName: a2Name.
	
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofMetaclassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofMetaclassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP3 ofMetaclassName: a2Name).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofMetaclassName: a2Name).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofMetaclassName: a2Name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofMetaclassName: a2Name).
	
	p2 removeSelector: #methodDefinedInP2 ofMetaclassName:  a2Name.
	self deny: (p2 includesDefinedSelector: #methodDefinedInP2 ofMetaclassName: a2Name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofMetaclassName: a2Name).
	
	p1 removeSelector: #methodDefinedInP1 ofMetaclassName:  a2Name.
	self deny: (p1 includesDefinedSelector: #methodDefinedInP3 ofMetaclassName: a2Name).
	self deny: (p1 includesExtensionSelector: #methodDefinedInP3 ofMetaclassName: a2Name).
	
	
	


! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 15:13'!
testBogusClassAddition
	"self debug: #testBogusClassAddition"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinitionName: a1 name.
	self assert: p definedClasses size = 1.
	self assert: (p includesClass: a1).
	self should: [p addClassDefinitionName: a1 class name] raise: Error.
! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 16:24'!
testClassAddition
	"self debug: #testClassAddition"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	self assert: p definedClasses size = 1.
	self assert: (p includesClass: a1).
	self assert: (p includesClass: a1 class)
! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 16:25'!
testClassDefinitionRemoval
	"self debug: #testClassDefinitionRemoval"

	| p a1 b1|
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	b1 := self createNewClassNamed: #B1InPAckageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	p addClassDefinition: b1.
	self assert: p definedClasses size = 2.
	self assert: (p includesClass: a1).
	self assert: (p includesClass: b1).
	p removeClassDefinition: a1.
	self assert: p definedClasses size = 1.
	self deny: (p includesClass: a1).
	self assert: (p includesClass: b1).
	p removeClassDefinition: b1 class.
	self deny: (p includesClass: b1).
! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 16:39'!
testClassDefinitionRemovalName
	"self debug: #testClassDefinitionRemovalName"

	| p a1 b1|
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	b1 := self createNewClassNamed: #B1InPAckageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinitionName: a1 name.
	p addClassDefinitionName: b1 name.
	self assert: p definedClasses size = 2.
	self assert: (p includesClass: a1).
	self assert: (p includesClass: b1).
	p removeClassDefinitionName: a1 name.
	self assert: p definedClasses size = 1.
	self deny: (p includesClass: a1).
	self assert: (p includesClass: b1).
	self should: [p removeClassDefinitionName: b1 class name] raise: Error.
	p removeClassDefinitionName: b1  name.
	self deny: (p includesClass: b1).
! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 4/21/2010 09:25'!
testClasses
	"self debug: #testClasses"
	
	| p1 p2 a2  b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (a2>>#methodPackagedInP1).
	b2 class compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (b2 class>>#methodPackagedInP1).
	
	self assert: p1 classes size = 2.
	self assert: p2 classes size = 2.
	
! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 16:41'!
testDefinedClassesAndDefinedClassNames
	"self debug: #testDefinedClassesAndDefinedClassNames"

	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	self assert: p definedClasses size = 1.
	self assert: (p definedClasses  includes: a1).
	self assert: (p definedClassNames includes: a1 name).
	
	b1 := self createNewClassNamed: #B1InPackageP1 inCategory: 'P1'.
	p addClassDefinition: b1.
	self assert: p definedClasses size = 2.
	self assert: (p definedClasses includes: b1).
	self assert: (p definedClassNames includes: b1 name).
	! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'sd 4/25/2010 17:38'!
testExtensionClassNames
	"self debug: #testExtensionClassNames"
	
	| p1 p2  a2 b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: b2).
	self assert: (p2 includesClass: a2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addSelector: #methodDefinedInP1 ofClassName: a2 name.
	
	self assert: p1 extensionSelectors size = 1.
	self assert: p1 extensionMethods size = 1.
	self assert: (p1 extendedClassNames includes: #A2InPackageP2).
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compile: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addSelector: #firstMethodInB2PackagedInP1 ofClassName: b2 name.
	
	self assert: p1 extensionSelectors  size = 2.
	self assert: p1 extensionMethods  size = 2.
	self assert: (p1 extendedClassNames includes: #B2InPackageP2).
	self deny: (p1 includesClass: b2).
	
	b2 compileSilently: 'secondMethodInB2PackagedInP1 ^ 2'.
	p1 addSelector: #secondMethodInB2PackagedInP1 ofClassName: b2 name.
	
	self assert: p1 extensionSelectors  size = 3.
	self assert: p1 extensionMethods  size = 3.
	self assert: p1 extendedClasses size = 2.
	self assert: (p1 extendedClassNames includes: #B2InPackageP2).
	self deny: (p1 includesClass: b2).! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'sd 4/25/2010 16:06'!
testExtensionClasses
	"self debug: #testExtensionClasses"
	
	| p1 p2 a2  b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: a2).
	
	a2 compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (a2>>#methodPackagedInP1).
	
	self assert: p1 extendedClasses size = 1.
	self assert: (p1 extendedClasses  includes: a2).
	self assert: p1 extendedClassNames size = 1.
	self assert: (p1 extendedClassNames  includes: a2 name).
	
	b2 class compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (b2 class>>#methodPackagedInP1).
	
	self assert: p1 extendedClasses size = 2.
	self assert: (p1 extendedClasses  includes: b2 class).
	"extensionClasses returns or metaclasses while extensionClassNames returns class names (but not metaclass names)"

	self assert: p1 extendedClassNames size = 2.
	self assert: (p1 extendedClassNames  includes: b2 name).
	
! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'sd 4/25/2010 17:45'!
testExtensionClassesWithCompiledMethod
	"self debug: #testExtensionClassesWithCompiledMethod"
	
	| p1 p2  a2 b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: b2).
	self assert: (p2 includesClass: b2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	self assert: p1 extensionMethods size = 1.
	self assert: p1 extensionSelectors size = 1.
	self assert: p1 extendedClasses size = 1.
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compile: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addMethod: (b2>>#firstMethodInB2PackagedInP1).
	
	self assert: p1 extensionSelectors  size = 2.
	self assert: p1 extensionMethods  size = 2.
	self assert: p1 extendedClasses size = 2.
	self deny: (p1 includesClass: b2).
	
	b2 compileSilently: 'secondMethodInB2PackagedInP1 ^ 2'.
	p1 addMethod: (b2>>#secondMethodInB2PackagedInP1).
	
	self assert: p1 extensionSelectors  size = 3.
	self assert: p1 extensionMethods  size = 3.
	self assert: p1 extendedClasses size = 2.
	self deny: (p1 includesClass: b2).! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'sd 4/25/2010 17:13'!
testExtensionMethods
	"self debug: #testExtensionMethods"
	
	| p1 p2 a2  b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	self assert: p1 extensionSelectors  size = 1.
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compileSilently: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addMethod: (b2>>#firstMethodInB2PackagedInP1).
	self assert: p1 extensionSelectors  size = 2.
	! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 3/11/2010 14:35'!
testIncludeClass
	"self debug: #testIncludeClass"
	
	| p1 p2 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	
	self deny: (p1 includesClass: a2).
	p1 addMethod: (a2>>#methodPackagedInP1).
	self deny: (p1 includesClass: a2).
	"We should declare the class explictly. Adding a method does not declare
	the class as defined. The reason is that like that the client controls the granularity 
	and moment of class registration."
	
	p1 addClassDefinition: a2.
	self assert: (p1 includesClass: a2).
	self assert: (p1 includesClassNamed: a2 name).
	
	! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 4/21/2010 09:29'!
testIncludeClassMore
	"self debug: #testIncludeClassMore"
	
	| p1 p2 p3 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
	
	self assert: (p2 includesClass: a2).
	self deny: (p1 includesClass: a2).
	self deny: (p3 includesClass: a2).! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 4/21/2010 09:29'!
testIncludeSelectorOfClass
	"self debug: #testIncludeSelectorOfClass"
	
	| p1 p2 p3 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
	
	"includesSelector checks both in defined and extension so we test both"
	self assert: (p2 includesSelector: #methodDefinedInP2 ofClass: a2).
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2 name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofClassName: a2 name).
	
	self deny: (p2 includesSelector: #methodDefinedInP3 ofClass: a2).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2 name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2 name).
	
	self deny: (p2 includesSelector: #methodDefinedInP1 ofClass: a2).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofClassName: a2 name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofClassName: a2 name).! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 4/21/2010 09:29'!
testIncludeSelectorOfMetaClass
	"self debug: #testIncludeSelectorOfMetaClass"
	
	| p1 p2 p3 a2  a2class |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2class := a2 class.
	a2class compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2class>>#methodDefinedInP2).
	a2class compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2class>>#methodDefinedInP1).
	a2class compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2class>>#methodDefinedInP3).
	
	"includesSelector checks both in defined and extension so we test both"
	self assert: (p2 includesSelector: #methodDefinedInP2 ofClass: a2class).
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofMetaclassName: a2 name).
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClass: a2class).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP2 ofMetaclassName: a2 name).
	
	self deny: (p2 includesSelector: #methodDefinedInP3 ofClass: a2).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofMetaclassName: a2 name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP3 ofMetaclassName: a2 name).
	
	self deny: (p2 includesSelector: #methodDefinedInP1 ofMetaclassName: a2).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofMetaclassName: a2 name).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofClass: a2 class).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofClassName: a2 name).
	self deny: (p3 includesDefinedSelector: #methodDefinedInP1 ofClass: a2).
! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'StephaneDucasse 4/21/2010 09:30'!
testIncludesMethodOfClassInPresenceOfOtherPackageExtensions
	"self debug: #testIncludesMethodOfClassInPresenceOfOtherPackageExtensions"
	
	| p1 p2 p3 a2  a2name |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	a2name := #A2InPackageP2.
	a2 := self createNewClassNamed: a2name inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2name).
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	self assert: (p1 includesExtensionSelector: #methodDefinedInP1 ofClassName: a2name).
	a2 compile: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2name).
		
	self assert: (p2 includesDefinedSelector: #methodDefinedInP2 ofClassName: a2name).
	self deny: (p2 includesDefinedSelector: #methodDefinedInP3 ofClassName: a2name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2name).
	self deny: (p2 includesDefinedSelector: #methodDefinedInP1 ofClassName: a2name).
	self deny: (p2 includesExtensionSelector: #methodDefinedInP1 ofClassName: a2name).
	
	
	! !

!RPackage2IncrementalTest methodsFor: 'test extension' stamp: 'sd 4/25/2010 17:46'!
testIncludesOrTouches
	"self debug: #testIncludesOrTouches"
	
	| p1 p2 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: a2).
	
	a2 compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (a2>>#methodPackagedInP1).
	
	self assert: p1 extensionMethods size = 1.
	self assert: p1 extensionSelectors size = 1.
	self deny: (p1 includesClass: a2).
	self assert: (p1 definesOrExtendsClass: a2).
	self assert: (p2 includesClass: a2).! !

!RPackage2IncrementalTest methodsFor: 'test method addition removal' stamp: 'StephaneDucasse 3/11/2010 16:50'!
testMethodAddition
	"self debug: #testMethodAddition"

	| p1 a1 methodRef |
	p1 := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	a1 compileSilently: 'foo ^ 10'.
	p1 addMethod:  (a1>>#foo).
	self assert: (p1 includesSelector: #foo ofClass: a1).
	! !

!RPackage2IncrementalTest methodsFor: 'test package belonging' stamp: 'sd 4/25/2010 17:33'!
testPackageOfClassForClassesNotDefinedInPackageButJustExtendingIt
	"self debug: #testPackageOfClassForClassesNotDefinedInPackageButJustExtendingIt"
	
	| p1 p2 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	self assert: a2 package = p2.
	p1 extensionMethods do: 
		[:each | self deny: each methodClass package = p1 ].
	"the package of a class which is extended inside a package p, is not p
	but the package where the class was defined"! !

!RPackage2IncrementalTest methodsFor: 'test package belonging' stamp: 'StephaneDucasse 3/11/2010 16:46'!
testPackageOfClassForDefinedClasses
	"self debug: #testPackageOfClassForDefinedClasses"
	
	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inPackage: p.
	b1 := self createNewClassNamed: #B1InPAckageP1 inPackage: p.

	self assert: a1 package = p.
	self assert: b1 package = p.
! !

!RPackage2IncrementalTest methodsFor: 'test registration' stamp: 'StephaneDucasse 3/11/2010 16:21'!
testRegisterUnregister
	"Verify that when we register a class, the package organizer register it."
	"self debug: #testRegisterUnregister"

	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPackageP1.
	p definedClassNames add: #A1InPackageP1.
		"ugly but necessary to test registerClass: independant from addClassDefinition:"
	p registerClass: a1.
	self assert: (p class packageOrganizerClass default packageOf: a1) = p.
	p definedClassNames remove: #A1InPackageP1.
	p unregisterClass: a1.
	self assert: (p class packageOrganizerClass default packageOf: a1) isNil.
	
	p definedClassNames add: #A1InPackageP1.
		"ugly but necessary to test registerClass: independant from addClassDefinition:"
	p registerClassName: a1 name.
	self assert: (p class packageOrganizerClass default packageOf: a1) = p.
	p definedClassNames remove: #A1InPackageP1.
	p unregisterClassName: a1 name.
	self assert: (p class packageOrganizerClass default packageOf: a1) isNil
	! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'sd 4/25/2010 15:40'!
testRemoveClassAfterMethods
	"self debug: #testRemoveClassAfterMethods"

	| p1 a1 b1 |
	p1 := self createNewPackageNamed: 'P1'.
	"the class is created but not added to the package for now"
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.
	p1 addClassDefinition: a1.
	self assert: p1 definedClasses size = 1.
	a1 compileSilently: 'newlyAddedToA1 ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#newlyAddedToA1).
	
	"now we add a new method to the class and the to the package: it is considered
	as defined"
	self deny: (p1 includesExtensionSelector: #newlyAddedToA1 ofClass: a1).
	self assert: (p1 includesDefinedSelector: #newlyAddedToA1 ofClass: a1).
	
	p1 removeClassDefinition: a1.
	self assert: p1 definedClasses size = 0.
	
	self assert: (p1 includesSelector: #newlyAddedToA1 ofClass: a1).
	self assert: (p1 includesExtensionSelector: #newlyAddedToA1 ofClass: a1).
	self deny: (p1 includesDefinedSelector: #newlyAddedToA1 ofClass: a1).
	
	"Now we add the class and this means that the method is not defined anymore but an extension "
	! !

!RPackage2IncrementalTest methodsFor: 'test class addition removal' stamp: 'StephaneDucasse 3/11/2010 16:43'!
testUniqueClassInDefinedClassesUsingAddClassDefinition
	"self debug: #testUniqueClassInDefinedClassesUsingAddClassDefinition"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	self assert: p definedClasses size = 1.
	p addClassDefinition: a1.
	self assert: p definedClasses size = 1.
	p addClassDefinition: a1 class.
	self assert: p definedClasses size = 1.

! !

RPackage2IncrementalTest subclass: #RPackageCompleteSetupButForModificationTest
	instanceVariableNames: 'p1 p2 p3 a1 b1 a2 a3 b2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackageCompleteSetupButForModificationTest methodsFor: 'setup' stamp: 'sd 5/19/2010 16:21'!
setUp

	super setUp.
	self packageClass packageOrganizerClass: RMockEmptyPackageOrganizer. 
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	a1 :=  self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	b1 := self createNewClassNamed: #B1DefinedInP1 inPackage: p1.
	a2 := self createNewClassNamed: #A2DefinedInP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2DefinedInP2 inPackage: p2.
	a3 := self createNewClassNamed: #A3DefinedInP3 inPackage: p3.
	
	a1 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#methodDefinedInP1).
	a1 compileSilently: 'anotherMethodDefinedInP1 ^ #anotherMethodDefinedInP1'.
	p1 addMethod: (a1>>#anotherMethodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).

	a2 class compileSilently: 'classSideMethodDefinedInP3 ^ #classSideMethodDefinedInP3'.
	p3 addMethod: (a2 class>>#classSideMethodDefinedInP3)

! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test addition' stamp: 'sd 4/22/2010 07:53'!
testAddMethod
	"self debug: #testAddMethod"
	
	self assert: (p1 includesSelector: #methodDefinedInP1 ofClass: a1).
	
	a1 compileSilently: 'zork ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#zork).
	self assert: (p1 includesSelector: #zork ofClass: a1).
	
	self assert: (p3 includesSelector: #classSideMethodDefinedInP3 ofClass: a2 class)
	! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test addition' stamp: 'sd 4/22/2010 07:59'!
testBasicRemoveClass
	"we remove a class we check that it is not in the package anymore"
	"self debug: #testBasicRemoveClass"	

	|  size |
	size := p1 definedClasses size. 
	self assert: size = 2.
	self assert: (p1 includesClass: a1).
	self assert: (p1 includesClass: a1 class).
	p1 removeClassDefinition: a1.
	self assert: p1 definedClasses size = (size - 1).
	self assert: (p1 includesClass: b1).
	self assert: (p1 includesClass: b1 class).
	self deny: (p1 includesClass: a1).
	self deny: (p1 includesClass: a1 class).
	p1 removeClassDefinition: b1.
	self assert: p1 definedClasses size = (size - 2).
	self deny: (p1 includesClass: b1).
	self deny: (p1 includesClass: b1 class).

! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test starting situation' stamp: 'sd 4/24/2010 22:31'!
testExtensionMethodOfClass
	"self debug: #testExtensionMethodOfClass"
	
	"P1 
		A1DefinedInP1>>methodDefinedInP1
		B1DefinedInP1	
		A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
	P2
		A2DefinedInP2>>methodDefinedInP2
		B2DefinedInB2	

	P3
		A3DefinedInP3	
		A2DefinedInP2>>methodDefinedInP3  (was *P2*)"
		
	self assert: a2 methodDict keys size = 3.
	"the class contains methods defined by several packages 
	however when looking with a package view we only see the package 
	extension."
		
	self assert: (p2 extensionMethodsForClass: a2) size = 0.
	self assert: (p1 extensionMethodsForClass: a1) size = 0.
	"show that extensionMethodsForClass: does works for methods of class
	defined in the package"
	
	self assert: (p1 extensionMethodsForClass: a2) size = 1.
	self assert: (p3 extensionMethodsForClass: a2) size = 1.! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test addition' stamp: 'sd 4/24/2010 22:28'!
testRemoveMethod
	"self debug: #testRemoveMethod"
	
	p1 removeMethod: a1 >> #methodDefinedInP1.
	self deny: (p1 includesSelector: #methodDefinedInP1 ofClass: a1).
	! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test addition' stamp: 'sd 4/24/2010 22:27'!
testRemoveMethodDoesNotRemoveFromClass
	"we remove a method so it is not in the package but also not from the class anymore"
	"self debug: #testRemoveMethodDoesNotRemoveFromClass"
	
	p1 removeMethod:  a1 >> #methodDefinedInP1.
	self deny: (p1 includesSelector: #methodDefinedInP1 ofClass: a1).
	self assert: (a1 selectors includes: #methodDefinedInP1)! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test addition' stamp: 'sd 4/24/2010 22:28'!
testRemoveShouldRaiseErrorWhenMethodIsNotInPackage
	"self debug: #testRemoveShouldRaiseErrorWhenMethodIsNotInPackage"
	
	self should: [p2 removeMethod: a2>>#methodDefinedZork] raise: Error! !

!RPackageCompleteSetupButForModificationTest methodsFor: 'test starting situation' stamp: 'sd 4/22/2010 07:52'!
testStartingSituation
	"self debug: #testStartingSituation"
	
	self deny: (p2 includesClass: b1).
	self assert: (p2 includesClass: b2).
	"a locally defined class not extended by other packages"
	
	self assert: (p2 includesClass: a2).  
	"a locally defined class extended by other packages"
	
	self assert: (p1 definesOrExtendsClass: a2).
	self deny: (p1 includesClass: a2).! !

RPackage2IncrementalTest subclass: #RPackageReadOnlyCompleteSetupTest
	instanceVariableNames: 'p1 p2 p3 a1 b1 a2 a3 b2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackageReadOnlyCompleteSetupTest commentStamp: 'sd 4/22/2010 07:49' prior: 0!
The tests takes as fixture the following situation and exercises the readonly queries.
We should be able to use a test resources to speed it up.

P1 
	A1DefinedInP1
	A1DefinedInP1>>methodDefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
P2
	A2DefinedInP2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (was *P2*)!

!RPackageReadOnlyCompleteSetupTest methodsFor: 'expectedfailures' stamp: 'StephaneDucasse 4/20/2010 21:50'!
expectedFailures

	^ #(testClassNamesAndExtensionSelectorsDo testClassNamesAndExtensionSelectorsDo testClassesAndExtensionMethodsDo testClassesAndExtensionMethodsDo)! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'setup' stamp: 'sd 5/19/2010 16:16'!
setUp

	super setUp.
	self packageClass packageOrganizerClass: RMockEmptyPackageOrganizer. 
	self packageClass packageOrganizerClass reset.
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	a1 :=  self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	b1 := self createNewClassNamed: #B1DefinedInP1 inPackage: p1.
	a2 := self createNewClassNamed: #A2DefinedInP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2DefinedInP2 inPackage: p2.
	a3 := self createNewClassNamed: #A3DefinedInP3 inPackage: p3.
	
	a1 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#methodDefinedInP1).
	a1 compileSilently: 'anotherMethodDefinedInP1 ^ #anotherMethodDefinedInP1'.
	p1 addMethod: (a1>>#anotherMethodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).

	a2 class compileSilently: 'classSideMethodDefinedInP3 ^ #classSideMethodDefinedInP3'.
	p3 addMethod: (a2 class>>#classSideMethodDefinedInP3)

! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test compiled method' stamp: 'StephaneDucasse 4/20/2010 22:11'!
testClassIsExtendedInPackage
	"self debug: #testClassIsExtendedInPackage"
	
	self deny: ((Smalltalk at: #A1DefinedInP1) isExtendedInPackage: p1).
	self assert: (p1 includesClass: (Smalltalk at: #A1DefinedInP1)).
	self deny: (p1 extendsClass: (Smalltalk at: #A1DefinedInP1)).
	self assert: ((Smalltalk at: #A2DefinedInP2) isExtendedInPackage: p1).
	self deny: (p1 includesClass: (Smalltalk at: #A2DefinedInP2)).
	self assert: (p1 extendsClass: (Smalltalk at: #A2DefinedInP2)).
	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test slice' stamp: 'StephaneDucasse 4/20/2010 21:53'!
testClassNamesAndExtensionSelectorsDo2
	"self debug: #testClassNamesAndExtensionSelectorsDo2"
	

	self assert: (String streamContents: [:stream |
		{p1 . p2 .p3} do: [:p |
			p classNamesAndExtensionSelectorsDo: [:aClassName :selector |
				stream nextPutAll: aClassName  ; nextPutAll: ' ' ; nextPutAll:  selector asString; cr ]]])
		= 'A2DefinedInP2 methodDefinedInP1
A2DefinedInP2 methodDefinedInP3
A2DefinedInP2_class classSideMethodDefinedInP3
'.
! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test slice' stamp: 'StephaneDucasse 4/20/2010 21:42'!
testClassesAndExtensionMethodsDo
	"self debug: #testClassesAndExtensionMethodsDo"
	
	| block |

	self assert: (String streamContents: [:stream |
		block := [ :aClass :selectors |
				stream nextPutAll: aClass name ; nextPutAll: ' '.
				selectors do: [ :selector | stream nextPutAll: selector asString].
				stream cr.].
		{p1 . p2 . p3} do: [ :p |
			p classExtensionSlicesDo: block.
			p metaclassExtensionSlicesDo: block.]]) 
		= 'A2DefinedInP2 methodDefinedInP1
A2DefinedInP2 methodDefinedInP3
A2DefinedInP2 classSideMethodDefinedInP3
'! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test compiled method' stamp: 'StephaneDucasse 4/20/2010 21:49'!
testCompiledMethodIsDefinedInPackage
	"self debug: #testCompiledMethodIsDefinedInPackage"
	
	self assert: (((Smalltalk at: #A1DefinedInP1) >> #methodDefinedInP1) isDefinedInPackage: p1).
	self deny: (((Smalltalk at: #A2DefinedInP2) >> #methodDefinedInP1) isDefinedInPackage: p1).
	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test compiled method' stamp: 'tg 3/16/2010 17:44'!
testCompiledMethodIsExtensionInPackage
	"self debug: #testCompiledMethodIsExtensionInPackage"
	
	self deny: (((Smalltalk at: #A1DefinedInP1) >> #methodDefinedInP1) isExtensionInPackage: p1).
	self assert: (((Smalltalk at: #A2DefinedInP2) >> #methodDefinedInP1) isExtensionInPackage: p1).
	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test compiled method' stamp: 'sd 5/21/2010 15:09'!
testCompiledMethodPackageFromOrganizer
	"self debug: #testCompiledMethodPackageFromOrganizer"
	| method |
	method := (Smalltalk at: #A1DefinedInP1) >> #methodDefinedInP1.
	self assert: (method packageFromOrganizer: RMockEmptyPackageOrganizer default) = p1.
	method := (Smalltalk at: #A2DefinedInP2) >> #methodDefinedInP1.
	self assert: (method packageFromOrganizer: RMockEmptyPackageOrganizer default) = p1
	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'sd 4/21/2010 15:57'!
testDefinedSelectorsForClass
	"self debug: #testDefinedSelectorsForClass"
	
	self assert: (p1 definedSelectorsForClass: a1) size = 2. 
	self assert: (p1 definedMethodsForClass: a1) size = 2. 
	self assert: ((p1 definedSelectorsForClass: a1) includes: #methodDefinedInP1).
	self assert: ((p1 definedSelectorsForClass: a1) includes: #anotherMethodDefinedInP1).
	self assert: ((p1 definedMethodsForClass: a1) includes: ((Smalltalk at: #A1DefinedInP1)>>#methodDefinedInP1)).
	self assert: (p1 definedSelectorsForClass: Object) isEmpty. 
	self assert: (p1 definedSelectorsForClass: Object class) isEmpty. 
! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test accessing' stamp: 'sd 4/21/2010 17:07'!
testExtendingPackagesOfClass
	"since a class can be extended by several packages, we want the list of packages that extend
	the class"
	
	"self debug: #testExtendingPackagesOfClass"
		
	| packages |
	packages := a2 extendingPackages.
	"a2 is extended by p1 and p3"
	self assert: packages size = 2. 
	self assert: (packages includes: p1).
	self deny: (packages includes: p2).
	self assert: (packages includes: p3).
	
	packages := a1 extendingPackages.
	self assert: packages isEmpty. ! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'StephaneDucasse 4/20/2010 22:05'!
testExtensionSelectorsForClass
	"self debug: #testExtensionSelectorsForClass"
	
	self assert: (p1 extensionSelectorsForClass: a2) size = 1. 
	self assert: ((p1 extensionSelectorsForClass: a2) includes: #methodDefinedInP1).
	self assert: ((p1 extensionMethodsForClass: a2) includes: (Smalltalk at: #A2DefinedInP2) >> #methodDefinedInP1).
	self assert: (p1 extensionSelectorsForClass: Object) isEmpty.
	self assert: (p1 extensionSelectorsForClass: Object class) isEmpty! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'tg 3/13/2010 13:48'!
testMetaclassHasExtensions
	"self debug: #testMetaclassHasExtensions"

	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClassName: a2  name).
	self assert: (p3 includesExtensionSelector: #methodDefinedInP3 ofClass: a2).
	self assert: (p3 includesExtensionSelector: #classSideMethodDefinedInP3 ofMetaclassName: a2 name).
	self assert: (p3 includesExtensionSelector: #classSideMethodDefinedInP3 ofClass: a2 class)

	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'sd 4/21/2010 16:38'!
testMethods
	"self debug: #testMethods"
	
	| m1 m3 |
	m1 := p1 methods.
	self assert: m1 size = 3. 
	self assert: (m1 includes: a1>>#methodDefinedInP1).
	self assert: (m1 includes: a1>>#anotherMethodDefinedInP1). 
	self assert: (m1 includes: a2>>#methodDefinedInP1).
	m3 := p3 methods.
	self assert: m3 size = 2.
	self assert: (m3 includes: a2>>#methodDefinedInP3).
	self assert: (m3 includes: a2 class>>#classSideMethodDefinedInP3)! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'sd 4/21/2010 15:57'!
testMethodsForClass
	"self debug: #testMethodsForClass"
	
	self assert: (p1 methodsForClass: a1) size = 2. 
	self assert: ((p1 methodsForClass: a1) includes: ((Smalltalk at: #A1DefinedInP1)>>#methodDefinedInP1)).
	self assert: ((p1 methodsForClass: a1) includes: ((Smalltalk at: #A1DefinedInP1)>>#anotherMethodDefinedInP1)).
	self assert: (p1 methodsForClass: b1) isEmpty. 
	self assert: (p1 methodsForClass: Object) isEmpty. 
	self assert: (p1 methodsForClass: Object class) isEmpty. 

	self assert: (p3 methodsForClass: a2) size = 1.
	self assert: ((p3 methodsForClass: a2) includes: ((Smalltalk at: #A2DefinedInP2)>>#methodDefinedInP3)).
	self assert: (p3 methodsForClass: a2 class) size = 1.
	self assert:  ((p3 methodsForClass: a2 class) includes: ((Smalltalk at: #A2DefinedInP2) class>>#classSideMethodDefinedInP3))	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test accessing' stamp: 'sd 4/21/2010 17:08'!
testPackagesOfClass
	"since a class can be extended by several packages, we want the complete list of packages that define or extend
	the class"
	"self debug: #testPackagesOfClass"
		
	| packages extending |
	packages := a2 packages.
	"a2 is extended by p1 and p3"
	extending := a2 extendingPackages.
	self assert: extending size = 2. 
	self assert: (extending includes: p1).
	self assert: (extending includes: p3).
	
	self assert: packages size = 3. 
	self assert: (packages includes: p1).
	self assert: (packages includes: p2).
	self assert: (packages includes: p3).
	
	
	
	
	
	
	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'sd 4/21/2010 16:13'!
testSelectors
	"self debug: #testSelectors"
	
	self assert: (p1 selectors) size = 2. 
	self assert: ((p1 selectors) includes: #methodDefinedInP1).
	self assert: ((p1 selectors) includes: #anotherMethodDefinedInP1). 

	self assert: p3 selectors size = 2.
	self assert: (p3 selectors includes: #methodDefinedInP3).
	self assert:  (p3 selectors includes: #classSideMethodDefinedInP3)	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'sd 4/21/2010 15:58'!
testSelectorsForClass
	"self debug: #testSelectorsForClass"
	
	self assert: (p1 selectorsForClass: a1) size = 2. 
	self assert: ((p1 selectorsForClass: a1) includes: #methodDefinedInP1).
	self assert: ((p1 selectorsForClass: a1) includes: #anotherMethodDefinedInP1).
	self assert: (p1 selectorsForClass: b1) isEmpty. 
	self assert: (p1 selectorsForClass: Object) isEmpty. 
	self assert: (p1 selectorsForClass: Object class) isEmpty. 

	self assert: (p3 selectorsForClass: a2) size = 1.
	self assert: ((p3 selectorsForClass: a2) includes: #methodDefinedInP3).
	self assert: (p3 selectorsForClass: a2 class) size = 1.
	self assert:  ((p3 selectorsForClass: a2 class) includes: #classSideMethodDefinedInP3)	! !

!RPackageReadOnlyCompleteSetupTest methodsFor: 'test situation' stamp: 'tg 3/13/2010 16:54'!
testStartingSituation
	"self debug: #testStartingSituation"
	
	self deny: (p2 includesClass: b1).
	self assert: (p2 includesClass: b2).
	"a locally defined class not extended by other packages"
	
	self assert: (p2 includesClass: a2).  
	"a locally defined class extended by other packages"
	
	self assert: (p1 definesOrExtendsClass: a2).
	self deny: (p1 includesClass: a2).! !

RPackageTest subclass: #RPackageOrganizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackageOrganizerTest methodsFor: 'setup' stamp: 'StephaneDucasse 5/15/2010 23:57'!
setUp

	super setUp.
	RMockEmptyPackageOrganizer reset.
	RMockEmptyPackageOrganizer fillUp.
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'StephaneDucasse 5/15/2010 23:57'!
testDefinedClassesInstanceAndMetaSideAPI

	| p1 |
	p1 := self createNewPackageNamed: 'P1'.
	RMockEmptyPackageOrganizer default registerPackage: p1.
	p1 addClassDefinition: Point.
	p1 addMethod: Point>>#x.
	p1 addMethod: Point class>>#x:y:. 
	self assert: RMockEmptyPackageOrganizer default packageNames size = 1.
	self assert: RMockEmptyPackageOrganizer default packages size = 1.
	self assert: RMockEmptyPackageOrganizer default packages first definedClasses size = 1.
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'StephaneDucasse 5/15/2010 23:57'!
testEmpty
	"self debug: #testEmpty"

	self assert: RMockEmptyPackageOrganizer default packageNames isEmpty
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'sd 5/21/2010 15:23'!
testRegisteredIsIncludedInPackageNames
	"self debug: #testRegisteredIsIncludedInPackageNames"

	| p1 p2 p3 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	RMockEmptyPackageOrganizer default registerPackage: p1.
	RMockEmptyPackageOrganizer default registerPackage: p2.
	RMockEmptyPackageOrganizer default registerPackage: p3.
	self assert: RMockEmptyPackageOrganizer default packageNames size = 3.
	self assert: (RMockEmptyPackageOrganizer default includesPackage: p1). 
	self assert: (RMockEmptyPackageOrganizer  default includesPackageNamed: p1 name). 
	
	{p1 . p2 . p3} do: [:each | self assert:  (RMockEmptyPackageOrganizer default packageNames includes: each name)]
		
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'StephaneDucasse 5/15/2010 23:58'!
testRegisteredIsThere
	"self debug: #testRegisteredIsThere"

	| p1 p2 p3 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	RMockEmptyPackageOrganizer default registerPackage: p1.
	RMockEmptyPackageOrganizer default registerPackage: p2.
	RMockEmptyPackageOrganizer default registerPackage: p3.
	self assert: RMockEmptyPackageOrganizer default packageNames size = 3.
	
	{p1 . p2 . p3} do: [:each | self assert:  (RMockEmptyPackageOrganizer default packageNames includes: each name)]
		
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'StephaneDucasse 5/15/2010 23:58'!
testRegisteredNumberOfPackageIsOk
	"self debug: #testRegisteredNumberOfPackageIsOk"

	| p1 p2 p3 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	RMockEmptyPackageOrganizer default registerPackage: p1.
	RMockEmptyPackageOrganizer default registerPackage: p2.
	RMockEmptyPackageOrganizer default registerPackage: p3.
	self assert: RMockEmptyPackageOrganizer default packageNames size = 3.
	RMockEmptyPackageOrganizer default unregisterPackage: p3.
	self assert: RMockEmptyPackageOrganizer default packageNames size = 2.
	! !

!RPackageOrganizerTest methodsFor: 'tests extending' stamp: 'StephaneDucasse 5/16/2010 11:01'!
testRegistrationExtendingPackages
	"self debug: #testRegistrationExtendingPackages"

	self assert: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) isEmpty.
	(RMockEmptyPackageOrganizer default 
					registerExtendingPackage:  RMockFilledPackageOrganizer pointRectangleInGraphElement 
					forClass: Quadrangle).
	self deny: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) isEmpty.
	self assert: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) anyOne name = #GraphElement.
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'StephaneDucasse 5/15/2010 23:58'!
testUnregisterBasedOnNames
	"self debug: #testUnregisterBasedOnNames"

	| p1 p2 p3 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	RMockEmptyPackageOrganizer default registerPackage: p1.
	RMockEmptyPackageOrganizer default registerPackage: p2.
	RMockEmptyPackageOrganizer default registerPackage: p3.
	self assert: RMockEmptyPackageOrganizer default packageNames size = 3.
	
	{p1 . p2 . p3} do: [:each | (RMockEmptyPackageOrganizer default unregisterPackageNamed: each name).
							self deny:  (RMockEmptyPackageOrganizer default packageNames includes: each name)]
		
	! !

!RPackageOrganizerTest methodsFor: 'tests extending' stamp: 'sd 5/19/2010 15:38'!
testUnregistrationExtendingPackages
	"self debug: #testUnregistrationExtendingPackages"

	| p |
	p := RMockFilledPackageOrganizer pointRectangleInGraphElement.
	self assert: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) isEmpty.
	(RMockEmptyPackageOrganizer default 
					registerExtendingPackage:  p
					forClass: Quadrangle).
	self deny: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) isEmpty.
	self assert: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) anyOne name = #GraphElement.
	(RMockEmptyPackageOrganizer default 
					unregisterExtendingPackage:  p
					forClass: Quadrangle).
	self assert: (RMockEmptyPackageOrganizer default extendingPackagesOf: Quadrangle) isEmpty.
	! !

Object subclass: #RPackage2
	instanceVariableNames: 'classDefinedSelectors metaclassDefinedSelectors classExtensionSelectors metaclassExtensionSelectors classes name'
	classVariableNames: 'PackageOrganizerClass'
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackage2 commentStamp: 'sd 4/25/2010 17:49' prior: 0!
A RPackage2 is a new implementation of package. Note that I does not touch classes but I'm just a layer on top of classes therefore removing a method from a package does not change the underlying class. 


Instance Variables
	classDefinedSelectors:		Dictionary
	classExtensionSelectors:		Dictionary
	metaclassDefinedSelectors:		Dictionary
	metaclassExtensionSelectors:		Dictionary
	packageName:		String
	classes: OrderedCollection of Symbols

The reason we use four dictionaries is that this way we only store #Point in both as the class and the metaclass are involved. We do not have to with Point_class problem when comparing defined and extended classes. The idea is taken from the RBSelectorEnvironment. 					
		
Class Invariant one:
	Classes should not contain metaclass name, but only class names
	
	

Implementation notes 
====================
This class went over 3 internal representations implementation. 
	- first: 	a list of class + two dictionaries: class * methods
	This was not good since we add to go all the time over the list of classes.
	- second: 4 dictionaries class * selectors 
	This was not good since we want to have defined classes without methods. 
	- third: 4 dictionaries + definedClasses

Originally I wanted to avoid to have a defined class list and I wanted to avoid to have to declare the class as defined. But this is not really good since
	- we will want to know if a class definition (without method for example is defined in a given package)
	- second this is easier to hook the packageOf: behavior (we register the package class when the class is 	added to the package).

defined classes information is redundant with the dictionary keys of defined methods but we would have to check and register the class to the packageOrganizer the first time a method is defined then also check on remove to unregister the class from the packageOrganizer.

Adding a method does not define the class as a defined package class. This has to be done explictly. The reason for this choice is that a class should register to the packageOrganizer and that I do not want to do it each time a method is added and I do not want to test it each time a method is added. Now this is done only when the class is declared as defined. 
We could also give the complete freedom to the client to register the class but I thought it was a good compromise. 
	
Todo
====
	Next: 
	- should use resources for the completeSetupTest
	- register to systemNotifier
	- build up a synchronizer that import PackageInfo.
	We should port all the tests.
	We should test packages and extendingPackages defined on classDescription
	We should add MC support.
	- add events.

	

	
!

!RPackage2 class methodsFor: 'initialize-release' stamp: 'StephaneDucasse 2/21/2010 17:30'!
initialize
	"self initialize"
	
	self packageOrganizerClass: RPackageOrganizer! !

!RPackage2 class methodsFor: 'instance creation' stamp: 'abc 5/9/2010 22:45'!
named: aString

	| newPackage |
	newPackage := self new name: aString ;yourself.
	PackageOrganizerClass default registerPackage: newPackage.
	^ newPackage! !

!RPackage2 class methodsFor: 'protected' stamp: 'StephaneDucasse 2/21/2010 17:31'!
packageOrganizerClass

	^ PackageOrganizerClass! !

!RPackage2 class methodsFor: 'protected' stamp: 'StephaneDucasse 2/21/2010 17:31'!
packageOrganizerClass: aPackageOrganizerClass

	PackageOrganizerClass := aPackageOrganizerClass! !

!RPackage2 methodsFor: 'class addition' stamp: 'sd 4/22/2010 07:58'!
addClassDefinition: aClass
	"Add only class definition, aClass, as defined classes of the receiver. The term classdefinition denotes the fact that we do not treat its methods. aClass methods are not added to the package. aClass can be either a class or a metaclass"
	
	self addClassDefinitionName: aClass theNonMetaClass name.
	! !

!RPackage2 methodsFor: 'class addition' stamp: 'sd 4/24/2010 22:47'!
addClassDefinitionName: aClassName
	"aClassName should only be a class (and not a metaclass name)"
	
	| aClassNameSymbol |
	('* class' match: aClassName) ifTrue: [^ self error: 'no metaclass name'].
	aClassNameSymbol := aClassName asSymbol.
	(classes includes: aClassNameSymbol)
		ifFalse: 
			[classes add: aClassNameSymbol.
			self extendedMethodsShouldBecomeDefinedWhenAddingClassName: aClassNameSymbol.
			self registerClassName: aClassNameSymbol]
	! !

!RPackage2 methodsFor: 'compiled method handling' stamp: 'sd 5/19/2010 15:58'!
addMethod: aCompiledMethod 
	"Add the method to the receiver as a defined method if the class is  defined in it, else as an extension."

	| mthClass |
	mthClass := aCompiledMethod methodClass. 
	(self includesClass: mthClass )
		ifFalse: [mthClass isMeta
					ifTrue: [(metaclassExtensionSelectors 
								at: mthClass theNonMetaClass name  
								ifAbsentPut: [Set new]) add: aCompiledMethod selector.]
					ifFalse: [(classExtensionSelectors 
								at: mthClass name 
								ifAbsentPut: [Set new]) add: aCompiledMethod selector].
				"we added a method extension so the receiver is an extending package of the class"
				PackageOrganizerClass default registerExtendingPackage: self forClass: mthClass]
		ifTrue: [mthClass isMeta
					ifTrue: [(metaclassDefinedSelectors 
								at: mthClass theNonMetaClass name  
								ifAbsentPut: [Set new]) add: aCompiledMethod selector]
					ifFalse: [(classDefinedSelectors 
								at: mthClass name 
								ifAbsentPut: [Set new]) add: aCompiledMethod selector]].	
	^ aCompiledMethod.! !

!RPackage2 methodsFor: 'selector handling' stamp: 'StephaneDucasse 4/20/2010 22:56'!
addSelector: aSelector ofClassName: aClassName
	"Add the method to the receiver. If the class is not locally defined in that package then the method is defined as a method extension: ie extending another package. Note that this method does not add the method to the class, it just records in the package that the method is packaged."
	
	( self includesClassNamed: aClassName)
		ifFalse: [(classExtensionSelectors at: aClassName asSymbol ifAbsentPut: [Set new]) add: aSelector]
		ifTrue: [(classDefinedSelectors at: aClassName asSymbol ifAbsentPut: [Set new]) add: aSelector].
	^ aSelector.! !

!RPackage2 methodsFor: 'selector handling' stamp: 'StephaneDucasse 4/20/2010 22:56'!
addSelector: aSelector ofMetaclassName: aClassName
	"Add the method to the receiver. If the class is not locally defined in that package then the method is defined as a method extension: ie extending another package. Note that this method does not add the method to the class, it just records in the package that the method is packaged. aClassName is the sole instance class name and not its metaclass one: i.e. adding Point class>>new is done as addSelector: #new ofMetaclassName: #Point"
	
	( self includesClassNamed: aClassName) 
		ifFalse: [(metaclassExtensionSelectors at: aClassName asSymbol ifAbsentPut: [Set new]) add: aSelector]
		ifTrue: [(metaclassDefinedSelectors at: aClassName asSymbol ifAbsentPut: [Set new]) add: aSelector].
	^ aSelector.! !

!RPackage2 methodsFor: 'slices' stamp: 'StephaneDucasse 3/11/2010 18:46'!
classDefinedSlicesDo: aBlock
	"This method iterates over the defined class and their associated selectors. a slice is a class * list of selectors. aBlock will be applied to all the extensions slices of the receiver. aBlok first argument is the class and the second argument a list of method selectors"
	
	classDefinedSelectors keysAndValuesDo: aBlock! !

!RPackage2 methodsFor: 'slices' stamp: 'StephaneDucasse 3/11/2010 18:46'!
classExtensionSlicesDo: aBlock
	"This method iterates over the class extensions and their associated selectors. A slice is a class * list of selectors. aBlock will be applied to all the extensions slices of the receiver. aBlok first argument is the class and the second argument a list of method selectors"
	
	classExtensionSelectors keysAndValuesDo: aBlock! !

!RPackage2 methodsFor: 'slices' stamp: 'StephaneDucasse 3/11/2010 18:40'!
classNamesAndExtensionSelectorsDo: aBlock
	"Iterate over the extended methods grouped by classes and apply the argument.
	The block will be passed a class name and each individual selectors.
	Only classes with methods are paired with their methods"
	
	classExtensionSelectors keysAndValuesDo: [ :classSymbol :methods |
			methods do: [ :sel |  aBlock value: classSymbol	value: sel]].
	
	metaclassExtensionSelectors keysAndValuesDo: [ :classSymbol :methods |
			methods do: [ :sel |  aBlock 
									value: (classSymbol, '_class') asSymbol
									value: sel]].	! !

!RPackage2 methodsFor: 'all accessing' stamp: 'sd 4/25/2010 16:03'!
classes
	"Return all the classes"
	
	^ self definedClasses, self extendedClasses 
	
	! !

!RPackage2 methodsFor: 'all accessing' stamp: 'StephaneDucasse 3/11/2010 15:17'!
definedClassNames 
	"Return the class names having methods defined in the receiver."
	
	^ classes! !

!RPackage2 methodsFor: 'all accessing' stamp: 'StephaneDucasse 5/15/2010 19:47'!
definedClasses 
	
	^ classes collect: [:each | Smalltalk globals at: each]
	! !

!RPackage2 methodsFor: 'iteration' stamp: 'StephaneDucasse 2/21/2010 17:25'!
definedClassesDo: aBlock

	classDefinedSelectors keysDo: aBlock.
	metaclassDefinedSelectors keysDo: aBlock.! !

!RPackage2 methodsFor: 'private' stamp: 'sd 4/25/2010 15:29'!
definedMethodsBecomeExtendedForClass: aClassName
	"the package may contain defined methods and their class is removed to the receiver. The status of such methods should now be extended"
	"Precondition: aClassName is in the defined list"
	
	| ext |
	(classDefinedSelectors keys includes: aClassName) ifFalse: [^ self].
	ext := classDefinedSelectors at: aClassName.
	classDefinedSelectors removeKey: aClassName.
	(classExtensionSelectors keys includes: aClassName) ifTrue: [self error: aClassName , ' should not be defined in extension'.]. "we should remove this check later"
	classExtensionSelectors at: aClassName put: ext.! !

!RPackage2 methodsFor: 'private' stamp: 'sd 4/25/2010 15:37'!
definedMethodsBecomeExtendedForMetaclass: aClassName
	"the package may contain defined methods and their class is removed to the receiver. The status of such methods should now be extended"
	"Precondition: aClassName is in the defined list"
	
	| ext |
	(metaclassDefinedSelectors keys includes: aClassName) ifFalse: [^ self].
	ext := metaclassDefinedSelectors at: aClassName.
	metaclassDefinedSelectors classDefinedSelectors removeKey: aClassName.
	(metaclassExtensionSelectors keys includes: aClassName) ifTrue: [self error: aClassName , ' should not be defined in extension'.]. "we should remove this check later"
	metaclassDefinedSelectors at: aClassName put: ext.! !

!RPackage2 methodsFor: 'accessing' stamp: 'tg 3/13/2010 14:17'!
definedMethodsForClass: aClass

	^ (self definedSelectorsForClass: aClass) collect: [ :each | aClass >> each ]! !

!RPackage2 methodsFor: 'private' stamp: 'sd 4/25/2010 15:41'!
definedMethodsShouldBecomeExtensionWhenRemovingClass: aClassName
	"the package may contain defined methods and their class is removed from the receiver. The status of such methods should now be extensions"
	
	self definedMethodsBecomeExtendedForClass: aClassName.
	self definedMethodsBecomeExtendedForMetaclass: aClassName.! !

!RPackage2 methodsFor: 'accessing' stamp: 'tg 3/13/2010 13:31'!
definedSelectorsForClass: aClass

	^ aClass isMeta
		ifTrue: [ metaclassDefinedSelectors at: aClass theNonMetaClass name ifAbsent: [ #() ] ]
		ifFalse: [ classDefinedSelectors at: aClass name ifAbsent: [ #() ] ]! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 4/20/2010 22:22'!
definesClass: aClass
	"Returns true if the class, aClass, is one of the locally defined classes"
	"should be probably removed since this is the same as includesClass: or the inverse"
	^ self includesClass: aClass! !

!RPackage2 methodsFor: 'testing' stamp: 'sd 4/25/2010 16:06'!
definesOrExtendsClass: aClass
	"Returns true whether the class, aClass, is one of the locally defined classes of the receiver or 
	if the receiver extends such class (that is defined in another package)"
	
	| canonizedName |
	canonizedName := aClass theNonMetaClass name.
	^ (classes includes: canonizedName) 
		or: [self extendedClassNames includes:  canonizedName]
! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 16:06'!
extendedClassNames 
	"Return the name of the classes which are extended by the receiver package. if a metaclass is extended, just get its sole instance class name."
	
	^ classExtensionSelectors keys union: metaclassExtensionSelectors keys! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 16:03'!
extendedClasses 
	"Return classes and metaclasses that are extended in the receiver"
	
	^ (metaclassExtensionSelectors keys collect: [:each | (Smalltalk at: each) class])
			union:  (classExtensionSelectors keys collect: [:each | Smalltalk at: each])! !

!RPackage2 methodsFor: 'private' stamp: 'sd 4/25/2010 15:29'!
extendedMethodsBecomeDefinedForClass: aClassName
	"the package may contain extension methods and their class is added to the receiver. The status of such methods should now be defined"
	"Precondition: aClassName is in the defined list"
	
	| ext |
	(classExtensionSelectors keys includes: aClassName) ifFalse: [^ self].
	ext := classExtensionSelectors at: aClassName.
	classExtensionSelectors removeKey: aClassName.
	(classDefinedSelectors keys includes: aClassName) ifTrue: [self error: aClassName , ' should not be defined in extension'.]. "we should remove this check later"
	classDefinedSelectors at: aClassName put: ext.! !

!RPackage2 methodsFor: 'private' stamp: 'StephaneDucasse 5/15/2010 22:22'!
extendedMethodsBecomeDefinedForMetaclass: aClassName
	"the package may contain extension methods and their class is added to the receiver. The status of such methods should now be defined"
	"Precondition: aClassName is in the defined list"
	
	| ext |
	(metaclassExtensionSelectors keys includes: aClassName) ifFalse: [^ self].
	ext := metaclassExtensionSelectors at: aClassName.
	metaclassExtensionSelectors removeKey: aClassName.
	(metaclassDefinedSelectors keys includes: aClassName) 
			ifTrue: [self error: aClassName , ' should not be defined in extension'.]. "we should remove this check later"
	metaclassDefinedSelectors at: aClassName put: ext.! !

!RPackage2 methodsFor: 'private' stamp: 'sd 4/25/2010 15:29'!
extendedMethodsShouldBecomeDefinedWhenAddingClassName: aClassName
	"the package may contain extension methods and their class is added to the receiver. The status of such methods should now be defined"
	"Precondition: aClassName is in the defined list"
	
	self extendedMethodsBecomeDefinedForClass: aClassName.
	self extendedMethodsBecomeDefinedForMetaclass: aClassName.
	! !

!RPackage2 methodsFor: 'testing' stamp: 'sd 4/25/2010 16:05'!
extendsClass: aClass
	"Returns true if the receiver extends aClass (that is defined in another package)"
	
	| canonizedName |
	canonizedName := aClass theNonMetaClass name.
	^ self extendedClassNames includes:  canonizedName! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 17:29'!
extensionMethods
	"extension methods are methods defined on classes that are not defined on the receiver"
	
	| mthds |
	mthds := OrderedCollection new.
	classExtensionSelectors keysAndValuesDo: [ :classSymbol :methods |
			methods do: [ :sel | mthds add: ((Smalltalk at: classSymbol) >>	sel)]].
	metaclassExtensionSelectors keysAndValuesDo: [ :classSymbol :methods |
			methods do: [ :sel | mthds add: ((Smalltalk at: classSymbol) >>	sel)]].	
	^ mthds				! !

!RPackage2 methodsFor: 'accessing' stamp: 'tg 3/13/2010 14:18'!
extensionMethodsForClass: aClass

	^ (self extensionSelectorsForClass: aClass) collect: [ :each | aClass >> each ]! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 17:07'!
extensionSelectors
	"extension methods are methods defined on classes that are not defined on the receiver"
	
	| classSels |
	classSels := Set new.
	classExtensionSelectors values do: [:each | classSels addAll: each].
	metaclassExtensionSelectors values do: [:each | classSels addAll: each].
	^ classSels.
	! !

!RPackage2 methodsFor: 'accessing' stamp: 'StephaneDucasse 4/21/2010 14:05'!
extensionSelectorsForClass: aClass

	^ aClass isMeta
		ifTrue: [ metaclassExtensionSelectors at: aClass theNonMetaClass name ifAbsent: [ #() ] ]
		ifFalse: [ classExtensionSelectors at: aClass name ifAbsent: [ #() ] ]! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 17:56'!
includesClass: aClass	
	"Returns true if the receiver includes aClass in the classes that are defined within it: only class definition are considered - not class extensions"
	
	^ self includesClassNamed: aClass theNonMetaClass name! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 4/20/2010 22:53'!
includesClassNamed: aSymbol
	"Returns true if the receiver includes class named aSymbol in the classes that are defined within it: only class definition are considered - not class extensions"
	
	| aRealSymbol |
	aRealSymbol := aSymbol asSymbol.
	^ classes includes: aSymbol
	! !

!RPackage2 methodsFor: 'testing' stamp: 'tg 3/13/2010 15:13'!
includesDefinedSelector: aSelector ofClass: aClass
	"Return true if the receiver includes the method of selector aSelector. Only checks methods defined in this package"
	
	| sels |
	sels := self definedSelectorsForClass: aClass.
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 18:53'!
includesDefinedSelector: aSelector ofClassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Only checks methods defined (not extended by other packages or package extensions)"
	
	| sels |
	sels := classDefinedSelectors at: aClassName ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 18:53'!
includesDefinedSelector: aSelector ofMetaclassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Only checks methods defined (not extended by other packages or package extensions)"
	
	| sels |
	sels := metaclassDefinedSelectors at: aClassName ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'tg 3/13/2010 13:47'!
includesExtensionSelector: aSelector ofClass: aClass
	"Return true if the receiver includes the method of selector aSelector. Only checks methods extending other packages"
	
	| sels |
	sels := self extensionSelectorsForClass: aClass.
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 18:54'!
includesExtensionSelector: aSelector ofClassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Only checks methods extending other packages"
	
	| sels |
	sels := classExtensionSelectors at: aClassName ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 18:54'!
includesExtensionSelector: aSelector ofMetaclassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Only checks methods extending other packages"
	
	| sels |
	sels := metaclassExtensionSelectors at: aClassName ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage2 methodsFor: 'testing' stamp: 'tg 3/13/2010 16:42'!
includesSelector: aSelector ofClass: aClass
	"Return true if the receiver includes the method of selector aSelector. Checks methods defined locally as well as extending other packages"
	
	^ aClass isMeta
		ifTrue: [ self includesSelector: aSelector ofMetaclassName: aClass theNonMetaClass name  ]
		ifFalse: [ self includesSelector: aSelector ofClassName: aClass name  ]
	
! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 2/21/2010 20:50'!
includesSelector: aSelector ofClassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Checks methods defined locally as well as extending other packages"
	
	^ (self includesDefinedSelector: aSelector ofClassName: aClassName)
		or: [(self includesExtensionSelector: aSelector ofClassName: aClassName)]! !

!RPackage2 methodsFor: 'testing' stamp: 'StephaneDucasse 3/11/2010 17:38'!
includesSelector: aSelector ofMetaclassName: aClassName
	"Return true if the receiver includes the method of selector aSelector. Checks methods defined locally as well as extending other packages"
	
	^ (self includesDefinedSelector: aSelector ofMetaclassName: aClassName)
		or: [(self includesExtensionSelector: aSelector ofMetaclassName: aClassName)]! !

!RPackage2 methodsFor: 'initialize-release' stamp: 'StephaneDucasse 3/11/2010 15:09'!
initialize
	super initialize.
	classes := Set new.
	metaclassExtensionSelectors := Dictionary new.
	classExtensionSelectors := Dictionary new.
	classDefinedSelectors := Dictionary new.
	metaclassDefinedSelectors := Dictionary new.! !

!RPackage2 methodsFor: 'slices' stamp: 'StephaneDucasse 3/11/2010 18:47'!
metaclassDefinedSlicesDo: aBlock
	"This method iterates over the defined class and their associated selectors. a slice is a class * list of selectors. aBlock will be applied to all the extensions slices of the receiver. aBlok first argument is the class and the second argument a list of method selectors"
	
	metaclassDefinedSelectors keysAndValuesDo: aBlock! !

!RPackage2 methodsFor: 'slices' stamp: 'StephaneDucasse 4/20/2010 21:42'!
metaclassExtensionSlicesDo: aBlock
	"This method iterates over the metaclass extensions and their associated selectors. A slice is a class * list of selectors. aBlock will be applied to all the extensions slices of the receiver. aBlok first argument is the class and second argument a list of selectors"
	
	metaclassExtensionSelectors keysAndValuesDo: aBlock
	
! !

!RPackage2 methodsFor: 'all accessing' stamp: 'sd 4/21/2010 16:36'!
methods

	| mths |
	mths := OrderedCollection new.
	metaclassExtensionSelectors keysAndValuesDo: [:key :val | val do: [:sel | mths add: ((Smalltalk at: key) class >> sel)]].
	classExtensionSelectors keysAndValuesDo: [:key :val |val do: [:sel | mths add: ((Smalltalk at: key) >> sel)]].
	metaclassDefinedSelectors keysAndValuesDo: [:key :val | val do: [:sel | mths add: ((Smalltalk at: key) class >> sel)]].
	classDefinedSelectors keysAndValuesDo: [:key :val | val do: [:sel | mths add: ((Smalltalk at: key) >> sel)]].
	^ mths! !

!RPackage2 methodsFor: 'accessing' stamp: 'StephaneDucasse 4/21/2010 14:05'!
methodsForClass: aClass
	"returns all the compiled for a class in the receiver without distinction of extensions or not"
	
	^ (self includesClass: aClass)
		ifFalse: [self extensionMethodsForClass: aClass]
		ifTrue: [self definedMethodsForClass: aClass]
	! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 15:52'!
name

	^ name! !

!RPackage2 methodsFor: 'accessing' stamp: 'sd 4/25/2010 15:52'!
name: aSymbol

	name := aSymbol asSymbol! !

!RPackage2 methodsFor: 'printing' stamp: 'sd 4/25/2010 15:51'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: self name.! !

!RPackage2 methodsFor: 'private-register' stamp: 'StephaneDucasse 3/11/2010 13:31'!
registerClass: aClass

	PackageOrganizerClass default 
		registerPackage: self forClass: aClass
	! !

!RPackage2 methodsFor: 'private-register' stamp: 'StephaneDucasse 3/11/2010 15:13'!
registerClassName: aClassNameSymbol

	PackageOrganizerClass default 
		registerPackage: self forClassName: aClassNameSymbol
	! !

!RPackage2 methodsFor: 'class addition' stamp: 'sd 4/25/2010 15:47'!
removeClassDefinition: aClass
	"Remove the class definition from the package but not its methods"
	
	| removed aClassName|
	aClassName := aClass theNonMetaClass name.
	removed := classes remove: aClassName ifAbsent: [nil].
	removed ifNotNil: [
			self definedMethodsShouldBecomeExtensionWhenRemovingClass: aClassName.
			self unregisterClass: aClass.] 
	! !

!RPackage2 methodsFor: 'class addition' stamp: 'sd 4/25/2010 15:46'!
removeClassDefinitionName: aClassName
	"remove the class definition from the package but not its method"
	
	| removed aClassNameSymbol |
	('* class' match: aClassName) ifTrue: [^ self error: 'no metaclass name'].
	aClassNameSymbol := aClassName asSymbol.
	removed := classes remove: aClassNameSymbol ifAbsent: [nil].
	removed ifNotNil: [
				self definedMethodsShouldBecomeExtensionWhenRemovingClass: aClassName.
				self unregisterClassName: aClassNameSymbol.] 
	! !

!RPackage2 methodsFor: 'compiled method handling' stamp: 'sd 5/19/2010 15:59'!
removeMethod: aCompiledMethod 
	"Remove the method to the receiver as a defined method."

	| mr mthClass |
	mthClass := aCompiledMethod methodClass. 
	(self includesClass: mthClass)
		ifFalse: [mthClass isMeta
					ifTrue: [(metaclassExtensionSelectors 
								at: mthClass theNonMetaClass name  
								ifAbsentPut: [Set new]) remove: aCompiledMethod selector.]
					ifFalse: [(classExtensionSelectors 
								at: mthClass name 
								ifAbsentPut: [Set new]) remove: aCompiledMethod selector].
				"we remove a method extension so the receiver is an extending package of the class"
				PackageOrganizerClass default unregisterExtendingPackage: self forClass: mthClass]
		ifTrue: [mthClass isMeta
					ifTrue: [(metaclassDefinedSelectors 
								at: mthClass theNonMetaClass name  
								ifAbsentPut: [Set new]) remove: aCompiledMethod selector]
					ifFalse: [(classDefinedSelectors 
								at: mthClass name 
								ifAbsentPut: [Set new]) remove: aCompiledMethod selector]].	
	^ aCompiledMethod.! !

!RPackage2 methodsFor: 'selector handling' stamp: 'StephaneDucasse 4/20/2010 22:57'!
removeSelector: aSelector ofClassName: aClassName
	"Remove the method in the package but does not propagate to the class itself.  Note that this method does not remove the method from the class, it just records in the package that the method is not in the package anymore."
	
	( self includesClassNamed: aClassName)
		ifFalse: [(classExtensionSelectors  at: aClassName ifAbsent: [ ^ Set new]) remove: aSelector.]
		ifTrue: [(classDefinedSelectors  at: aClassName ifAbsent: [ ^ Set new]) remove: aSelector.].
	
	! !

!RPackage2 methodsFor: 'selector handling' stamp: 'StephaneDucasse 4/20/2010 22:57'!
removeSelector: aSelector ofMetaclassName: aClassName
	"Remove the method in the package. Note that this method does not remove the method from the class, it just records in the package that the method is not in the package anymore. aClassName is the sole instance class name and not its metaclass one: i.e. adding Point class>>new is done as removeSelector: #new ofMetaclassName: #Point"
	
	( self includesClassNamed: aClassName)
		ifFalse: [(metaclassExtensionSelectors  at: aClassName ifAbsent: [ ^ Set new]) remove: aSelector.]
		ifTrue: [(metaclassDefinedSelectors  at: aClassName ifAbsent: [ ^ Set new]) remove: aSelector.].
	
	! !

!RPackage2 methodsFor: 'all accessing' stamp: 'sd 4/21/2010 16:12'!
selectors

	| sel |
	sel := Set new.
	metaclassExtensionSelectors valuesDo: [:each | sel addAll: each].
	classExtensionSelectors valuesDo: [:each | sel addAll: each].
	metaclassDefinedSelectors valuesDo: [:each | sel addAll: each].
	classDefinedSelectors valuesDo: [:each | sel addAll: each].
	^ sel
	! !

!RPackage2 methodsFor: 'accessing' stamp: 'StephaneDucasse 4/21/2010 13:56'!
selectorsForClass: aClass
	"returns all the selector for a class in the receiver without distinction of extensions or not"
	
	^ (self includesClass: aClass)
		ifFalse: [self extensionSelectorsForClass: aClass]
		ifTrue: [self definedSelectorsForClass: aClass]
	! !

!RPackage2 methodsFor: 'private-register' stamp: 'StephaneDucasse 3/11/2010 13:31'!
unregisterClass: aClass

	 PackageOrganizerClass default 
		unregisterPackage: self forClass: aClass.! !

!RPackage2 methodsFor: 'private-register' stamp: 'StephaneDucasse 3/11/2010 15:14'!
unregisterClassName: aClassNameSymbol

	 PackageOrganizerClass default 
		unregisterPackage: self forClassName: aClassNameSymbol.! !

Object subclass: #RPackageBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackageBenchmark class methodsFor: 'as yet unclassified' stamp: 'tg 6/5/2010 11:45'!
initializeRepository

	[	RPackage2 packageOrganizerClass: RMockPackageOrganizer.
		RMockPackageOrganizer reset ] 
	ensure: [ RPackage2 packageOrganizerClass: RPackageOrganizer ]! !

!RPackageBenchmark class methodsFor: 'as yet unclassified' stamp: 'tg 6/5/2010 11:47'!
run
	" self run "
	| classes |
	[
		RPackage2 packageOrganizerClass: RMockPackageOrganizer.
		RMockPackageOrganizer reset.
		classes := Object withAllSubclasses.
		classes remove: BogusInfo.
		classes remove: UndefinedObject.
		classes remove: ClassTrait.
		classes remove: ClassTrait class.
		classes remove: GRPackage.
		classes := classes reject: [:each |  'MC*' match: each name].
		Time millisecondsToRun: [
			classes do: [:each | 
				each package.
				each extendingPackages. 
				each package selectorsForClass: each.
				each extendingPackages do: [ :package | 
					package selectorsForClass: each.]. 
				each methods do: [:method | 
					method packageFromOrganizer: RMockPackageOrganizer default
					]
			]
		].
	] ensure: [ RPackage2 packageOrganizerClass: RPackageOrganizer ]! !

Object subclass: #RPackageOrganizer
	instanceVariableNames: 'classPackageMapping packages classExtendingPackagesMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

RPackageOrganizer class
	instanceVariableNames: 'default'!

!RPackageOrganizer commentStamp: 'StephaneDucasse 5/15/2010 20:08' prior: 0!
A RPackageOrganizer is responsible for providing all the package currently defined in the system.
In addition it provides a back pointer from the class to its package.

The classPackageMapping and the classExtendingPackageMapping should be moved in the future to the classes
themselves. 

self systemChangeNotifier notify: self ofAllSystemChangesUsing: #callthismethodOnSelf.!

RPackageOrganizer class
	instanceVariableNames: 'default'!

RPackageOrganizer subclass: #RMockEmptyPackageOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RMockEmptyPackageOrganizer class methodsFor: 'class initialization' stamp: 'StephaneDucasse 5/15/2010 23:56'!
default
	
	default isNil ifTrue: [
		default := self new.
		default class fillUp.
		default ].
	^ default! !

!RMockEmptyPackageOrganizer class methodsFor: 'class initialization' stamp: 'StephaneDucasse 5/15/2010 23:56'!
fillUp
	"self fillUp"
	
	"does nothing"! !

!RMockEmptyPackageOrganizer class methodsFor: 'class initialization' stamp: 'StephaneDucasse 5/15/2010 23:56'!
reset

	default := nil! !

RMockEmptyPackageOrganizer subclass: #RMockFilledPackageOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RMockFilledPackageOrganizer commentStamp: 'StephaneDucasse 5/15/2010 23:55' prior: 0!
A RMockFilledPackageOrganizer is prefilled simple packageOrganizer for tests!

!RMockFilledPackageOrganizer class methodsFor: 'initialize-release' stamp: 'StephaneDucasse 5/15/2010 22:15'!
fillUp
	"self fillUp"
	
	self pointRectangleInGraphElement.
	self stringCharacterInCollPackage! !

!RMockFilledPackageOrganizer class methodsFor: 'initialize-release' stamp: 'StephaneDucasse 5/16/2010 00:01'!
pointRectangleInGraphElement
	"self pointRectangleInGraphElement"
	
	| previous p |
	[ previous := RPackage2 packageOrganizerClass.
	RPackage2 packageOrganizerClass: self.
	p := RPackage2 named: #GraphElement.
	p addClassDefinition: Point.
	p addMethod: Point>>#x.
	p addMethod: Point>>#rotateBy:about:. 
	p addClassDefinition: Rectangle.
	p addMethod: Quadrangle>>#intersect: ] ensure: [RPackage2 packageOrganizerClass: previous].
	^ p! !

!RMockFilledPackageOrganizer class methodsFor: 'initialize-release' stamp: 'StephaneDucasse 4/21/2010 08:52'!
stringCharacterInCollPackage
	"self stringCharacterInCollPackage"
	
	| previous p |
	[ previous := RPackage2 packageOrganizerClass.
	RPackage2 packageOrganizerClass: self.
	p := RPackage2 named: #CollPackage.
	p addClassDefinition: String.
	p addClassDefinition: Character.
	p addMethod: String>>#indexOf:.
	p addMethod: String>>#isAllDigits.
	p addMethod: String class>>#cr.
	p addMethod: Character>>#asInteger.
	p addMethod: Text>>#allBold] ensure: [RPackage2 packageOrganizerClass: previous]! !

RPackageOrganizer subclass: #RMockPackageOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RMockPackageOrganizer commentStamp: 'StephaneDucasse 5/15/2010 23:54' prior: 0!
A RMockPackageOrganizer is a mock to copy completely the complete package info and check
Normally should not be used in tests.
!

!RMockPackageOrganizer class methodsFor: 'singleton' stamp: 'stephane.ducasse 8/10/2009 11:35'!
reset

	default := nil! !

!RPackageOrganizer class methodsFor: 'singleton' stamp: 'StephaneDucasse 5/15/2010 19:33'!
default
	
	default isNil ifTrue: [
		default := self new.
		default initializeFromSystem.
		default ].
	^ default! !

!RPackageOrganizer class methodsFor: 'singleton' stamp: 'sd 5/19/2010 23:01'!
fillUp
	"self fillUp"
	| package |
	PackageOrganizer default packages do: [ :info |
			package := RPackage2 named: info name.
			info classes do: [ :each | package addClassDefinition: each ].
			info coreMethods do: [ :each |
				each isValid
					ifTrue: [ package addMethod: each compiledMethod ] ].
			info extensionMethods do: [ :each |
				each isValid
					ifTrue: [ package addMethod: each compiledMethod ] ] ]
		displayingProgress: 'Importing'! !

!RPackageOrganizer methodsFor: 'update' stamp: 'sd 5/21/2010 14:23'!
event: anEvent
	Transcript show: '>>>>>>>>>' ; show: anEvent; cr! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'sd 5/21/2010 14:56'!
extendingPackagesOf: aClass
	"Returns the packages extending the class aClass"
	^ classExtendingPackagesMapping at: aClass theNonMetaClass name ifAbsent: [#()]! !

!RPackageOrganizer methodsFor: 'testing' stamp: 'sd 5/21/2010 15:21'!
includesClass: aClass

	^ self includesClassNamed: aClass name! !

!RPackageOrganizer methodsFor: 'testing' stamp: 'sd 5/21/2010 15:18'!
includesClassNamed: aSymbol

	^ classPackageMapping keys includes: aSymbol ! !

!RPackageOrganizer methodsFor: 'testing' stamp: 'sd 5/21/2010 15:17'!
includesPackage: aPackage

	^ self includesPackageNamed: aPackage name! !

!RPackageOrganizer methodsFor: 'testing' stamp: 'sd 5/21/2010 15:16'!
includesPackageNamed: aSymbol

	^ packages keys includes:  aSymbol ! !

!RPackageOrganizer methodsFor: 'initialize' stamp: 'StephaneDucasse 5/15/2010 22:22'!
initialize
	super initialize.
	packages := IdentityDictionary new.
	classPackageMapping := IdentityDictionary new.
	classExtendingPackagesMapping := IdentityDictionary new.
	self register! !

!RPackageOrganizer methodsFor: 'initialize' stamp: 'sd 5/21/2010 13:20'!
initializeFromSystem
	| package |
	MCWorkingCopy allManagers
		do: [ :workingCopy |
			| info |
			info := workingCopy packageInfo.
			package := self registerPackageNamed: info packageName.
			info classes 
				do: [ :each | package addClassDefinition: each ].
			info coreMethods do: [ :each |
				each isValid
					ifTrue: [ package addMethod: each compiledMethod ] ].
			info extensionMethods do: [ :each |
				each isValid
					ifTrue: [ package addMethod: each compiledMethod ] ] ]
		displayingProgress: 'Importing from MC managers'! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'tg 3/13/2010 16:39'!
packageDefiningOrExtendingMethod: aCompiledMethod
	^ self packageDefiningOrExtendingSelector: aCompiledMethod selector inClass: aCompiledMethod methodClass
! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'tg 6/5/2010 11:38'!
packageDefiningOrExtendingSelector: aSelector inClass: aClass
	"this implementation is slower
		aClass packages detect: [:each | each includesSelector: aSelector ofClass: aClass ]"

	^ aClass packages detect: [:each | each includesSelector: aSelector ofClass: aClass ]
	
"	self packages detect: [:each | 
		each includesSelector: aSelector ofClass: aClass ].
"	
	! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'stephane.ducasse 7/16/2009 23:17'!
packageNames
	^ packages keys! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'tg 3/13/2010 16:28'!
packageOf: aClass

	self flag: #shouldProbablyReturnUnunpackagedPackage.
	^ classPackageMapping at: aClass theNonMetaClass name ifAbsent: [nil]! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'stephane.ducasse 7/10/2009 11:27'!
packages
	
	^ packages values! !

!RPackageOrganizer methodsFor: 'initialize' stamp: 'sd 5/21/2010 14:34'!
register
	MCWorkingCopy addDependent: self.
	
	"SystemChangeNotifier uniqueInstance notify: self ofAllSystemChangesUsing: #event:"! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'sd 5/21/2010 14:44'!
registerExtendingPackage: aPackage forClass: aClass 

	| cur |
	cur := classExtendingPackagesMapping at: aClass theNonMetaClass name ifAbsent: [nil].
	cur isNil
		ifTrue: [classExtendingPackagesMapping at: aClass theNonMetaClass name put: (Set with: aPackage)]
		ifFalse: [cur add: aPackage]! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'abc 5/9/2010 22:40'!
registerPackage: aPackage
	"Transcript show: self class printString , ' registerPackage: ', aPackage packageName asString ; cr."
	^ packages at: aPackage name asSymbol put: aPackage.
	! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'TestRunner 12/25/2009 22:13'!
registerPackage: aPackage forClass: aClass 
	(aPackage includesClass: aClass) 
		ifFalse: 
			[self error: aPackage name , ' does not includes the class ' , aClass name].
	^classPackageMapping at: aClass theNonMetaClass name put: aPackage! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'TestRunner 12/3/2009 21:42'!
registerPackage: aPackage forClassName: aClassNameSymbol
	(aPackage includesClassNamed: aClassNameSymbol asSymbol) 
		ifFalse: 
			[self error: aPackage name , ' does not includes the class ' , aClassNameSymbol].
	^classPackageMapping at: aClassNameSymbol put: aPackage! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'abc 5/9/2010 22:40'!
registerPackageNamed: aString
	^ RPackage2 named: aString! !

!RPackageOrganizer methodsFor: 'private' stamp: 'sd 5/21/2010 14:51'!
stopNotification
	"(self instVarNamed: #default) stopNotification"
	
	"pay attention that we can break the system using this method"
	
	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self.
	MCWorkingCopy removeDependent: self.! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'StephaneDucasse 12/25/2009 19:47'!
unregisterClass: aClass 
	
	^ classPackageMapping removeKey: aClass theNonMetaClass name asSymbol
	! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'sd 5/19/2010 16:13'!
unregisterExtendingPackage: aPackage forClass: aClass
	
	| cur |
	cur := classExtendingPackagesMapping at: aClass theNonMetaClass name ifAbsent: [nil].
	cur isNil
		ifFalse: [cur remove: aPackage ifAbsent: [] "not happy with this one"]! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'StephaneDucasse 5/16/2010 17:31'!
unregisterExtendingPackage: aPackage forClassName: aClassName
	self halt.! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'sd 4/25/2010 15:51'!
unregisterPackage: aPackage
	"Transcript show: self class printString , ' unregisterPackage: ', aPackage packageName asString ; cr."
	
	packages removeKey: aPackage name asSymbol ifAbsent: [].	
	! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'sd 5/19/2010 15:46'!
unregisterPackage: aPackage forClass: aClass 

	^classPackageMapping removeKey: aClass theNonMetaClass name! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'TestRunner 12/25/2009 22:03'!
unregisterPackage: aPackage forClassName: aClassName
	(aPackage includesClassNamed: aClassName asSymbol) 
		ifTrue: 
			[self error: aPackage name , ' still includes the class ' , aClassName asSymbol].
	^classPackageMapping removeKey: aClassName asSymbol! !

!RPackageOrganizer methodsFor: 'registration' stamp: 'StephaneDucasse 12/20/2009 11:22'!
unregisterPackageNamed: aSymbolName
	packages removeKey: aSymbolName asSymbol ifAbsent: [].	
	! !

!RPackageOrganizer methodsFor: 'update' stamp: 'sd 5/21/2010 14:49'!
update: anArray
	Transcript show: 'From MCWorking copy'; cr.
	super update: anArray.
	(anArray isArray and: [ anArray first = #registered ])
		ifTrue: [ self registerPackageNamed: anArray second name ].
	(anArray isArray and: [ anArray first = #unregistered ])
		ifTrue: [ self unregisterPackageNamed: anArray second name ]! !
RPackage2 initialize!
